import base64
import site
import pytz
import os

from datetime import datetime
from lxml import etree
from onvif import ONVIFCamera

from onvif_proxy.auth_utils import make_digest


# noinspection HttpUrlsUsage
class Camera:

    def __init__(self, name, host, port, username, password: str | bytes):
        self.name = name
        self.host = host
        self.port = port
        self.username = username

        if isinstance(password, str):
            self.password = password.encode('utf-8')
        elif isinstance(password, bytes):
            self.password = password
        else:
            raise TypeError

        wsdl_dir = os.path.join(site.getsitepackages()[0], 'onvif/wsdl')
        self.onvif_camera = ONVIFCamera(host, port, username, password, wsdl_dir)
        self.media_service = None
        self.profiles = None

    async def update(self):
        await self.onvif_camera.update_xaddrs()
        self.media_service = await self.onvif_camera.create_media_service()
        self.profiles = await self.media_service.GetProfiles()

    def make_digest(self):
        timestamp = datetime.utcnow().replace(tzinfo=pytz.utc, microsecond=0).isoformat()
        nonce = os.urandom(16)
        digest = make_digest(self.password, nonce, timestamp)
        nonce = base64.b64encode(nonce).decode('utf-8')
        return digest, nonce, timestamp

    def make_security_etree(self):
        nsmap = {
            'wsse': "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-secext-1.0.xsd",
            'wsu': "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-wssecurity-utility-1.0.xsd"
        }

        # Creating a Security Element
        security_elem = etree.Element("{%s}Security" % nsmap['wsse'])

        # Creating a UsernameToken Element
        username_token_elem = etree.SubElement(security_elem, "{%s}UsernameToken" % nsmap['wsse'])

        # Creating a Username element and adding text
        username_elem = etree.SubElement(username_token_elem, "{%s}Username" % nsmap['wsse'])
        username_elem.text = self.username

        digest, nonce, timestamp = self.make_digest()

        # Create a Password element and add a Type attribute and text
        password_elem = etree.SubElement(username_token_elem, "{%s}Password" % nsmap['wsse'])
        password_elem.attrib['Type'] = \
            "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-username-token-profile-1.0#PasswordDigest"
        password_elem.text = digest

        # Create a Nonce element and add an EncodingType attribute and text
        nonce_elem = etree.SubElement(username_token_elem, "{%s}Nonce" % nsmap['wsse'])
        nonce_elem.attrib['EncodingType'] = \
            "http://docs.oasis-open.org/wss/2004/01/oasis-200401-wss-soap-message-security-1.0#Base64Binary"
        nonce_elem.text = nonce

        # Create a Created element and add a namespace
        created_elem = etree.SubElement(username_token_elem, "{%s}Created" % nsmap['wsu'])
        created_elem.text = timestamp

        return security_elem
