from typing import List

from fastapi import APIRouter, HTTPException, status, Depends

from database import db_connection as db
from database.repositories.cam_groups import GroupRepository
from database.repositories.users import UserRepository
from database.schemas.cam_groups import GroupFull, GroupBase
from database.schemas.cams import CamOut
from database.repositories.nodes import NodeRepository

db_cam_groups_router = APIRouter()


@db_cam_groups_router.get("/list", response_model=List[GroupFull])
async def list_groups(
        group_repo: GroupRepository = Depends(db.get_repository(GroupRepository))
):
    groups = await group_repo.list_all()
    if groups is None:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND
        )
    return groups


@db_cam_groups_router.get("/{group_id}", response_model=GroupFull)
async def get_group(
        group_id: int,
        group_repo: GroupRepository = Depends(db.get_repository(GroupRepository))
):
    group = await group_repo.get(group_id)
    if group is None:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND
        )
    return group


@db_cam_groups_router.post("", response_model=GroupFull)
async def create_group(
    group: GroupBase,
    group_repo: GroupRepository = Depends(db.get_repository(GroupRepository))
):
    new_group = await group_repo.create(group)
    if new_group is None:
        raise HTTPException(
            status_code=status.HTTP_304_NOT_MODIFIED
        )
    return new_group


@db_cam_groups_router.patch("/{group_id}", response_model=GroupFull)
async def update_group(
    group_id: int,
    group: GroupBase,
    group_repo: GroupRepository = Depends(db.get_repository(GroupRepository))
):
    updated_group = await group_repo.update(group_id, group)
    if updated_group is None:
        raise HTTPException(
            status_code=status.HTTP_304_NOT_MODIFIED
        )

    return updated_group


@db_cam_groups_router.delete("/{group_id}", response_model=GroupFull)
async def delete_group(
        group_id: int,
        group_repo: GroupRepository = Depends(db.get_repository(GroupRepository))
):
    group = await group_repo.delete(group_id)
    if group is None:
        raise HTTPException(
            status_code=status.HTTP_304_NOT_MODIFIED
        )
    return group


@db_cam_groups_router.get("/{group_id}/cams", response_model=List[CamOut], tags=["Cams"])
async def list_cams_in_group(
        group_id: int,
        group_repo: GroupRepository = Depends(db.get_repository(GroupRepository))
):
    group = await group_repo.get(group_id)
    if group is None:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND
        )
    return await group_repo.list_cams_in_group(group)


@db_cam_groups_router.get("/user/{user_id}", response_model=List[GroupFull], tags=["Users"])
async def list_user_groups(
        user_id: int,
        user_repo: UserRepository = Depends(db.get_repository(UserRepository)),
        group_repo: GroupRepository = Depends(db.get_repository(GroupRepository))
):
    user = await user_repo.get(user_id)
    groups = await group_repo.get_user_groups(user)
    if groups is None:
        raise HTTPException(
            status_code=status.HTTP_404_NOT_FOUND
        )
    return groups


@db_cam_groups_router.post("/user/{user_id}", response_model=GroupFull, tags=["Users"])
async def add_user_group(
        user_id: int,
        group_id: int,
        user_repo: UserRepository = Depends(db.get_repository(UserRepository)),
        group_repo: GroupRepository = Depends(db.get_repository(GroupRepository))
):
    user = await user_repo.get(user_id)
    group = await group_repo.get(group_id)
    await group_repo.add_user_group(user, group)
    return group


@db_cam_groups_router.delete("/user/{user_id}", response_model=GroupFull, tags=["Users"])
async def delete_cam_group(
        user_id: int,
        group_id: int,
        user_repo: UserRepository = Depends(db.get_repository(UserRepository)),
        group_repo: GroupRepository = Depends(db.get_repository(GroupRepository))
):
    user = await user_repo.get(user_id)
    group = await group_repo.get(group_id)
    await group_repo.delete_user_group(user, group)
    return group
