from typing import List, Tuple

from fastapi import HTTPException
from sqlalchemy import select
from sqlalchemy.ext.asyncio import AsyncSession
from starlette import status

from database.models.cams import Cam, CamInGroup, Group
from database.models.nodes import Node
from database.repositories.base import Repository
from database.schemas.nodes import NodeUpdate, NodeCreate


# noinspection DuplicatedCode
class NodeRepository(Repository):
    def __init__(self, db_session: AsyncSession):
        super().__init__(db_session)

    async def list_all(self) -> List[Node]:
        query = select(Node)
        node_db_obj = await self._session.execute(query)
        return [node for node, in node_db_obj]

    async def get_or_none(self, node_id: int) -> Node | None:
        node_db_obj = await self._session.get(Node, node_id)
        return node_db_obj

    async def get(self, node_id: int) -> Node:
        node_db_obj = await self.get_or_none(node_id)
        if node_db_obj is None:
            raise HTTPException(
                status_code=status.HTTP_404_NOT_FOUND,
                detail=f"Node with id={node_id} not found"
            )
        return node_db_obj

    async def create(self, node: NodeCreate) -> Node:
        node_db_obj = Node(**node.model_dump(exclude_unset=True))
        self._session.add(node_db_obj)
        await self._session.commit()
        await self._session.refresh(node_db_obj)
        return node_db_obj

    async def update(self, node_id: int, node: NodeUpdate) -> Node | None:
        node_db_obj = await self._session.get(Node, node_id)
        for key, value in node.model_dump(exclude_unset=True).items():
            setattr(node_db_obj, key, value)

        await self._session.commit()
        await self._session.refresh(node_db_obj)
        return node_db_obj

    async def delete(self, node_id: int) -> Node | None:
        node_db_obj = await self._session.get(Node, node_id)
        await self._session.delete(node_db_obj)
        await self._session.commit()
        return node_db_obj

    async def get_node_cams(self, node: Node) -> List[Cam]:
        query = select(Cam).join(CamInGroup, CamInGroup.cam_id == Cam.id)
        query = query.filter(CamInGroup.node_id == node.id)
        cams_db_obj = await self._session.execute(query)
        return [cam for cam, in cams_db_obj]

    async def get_node_cam_pairs(self, group: Group) -> List[Tuple[int, int]]:
        query = select(CamInGroup.c.node_id, CamInGroup.c.cam_id)
        query = query.filter(CamInGroup.group_id == group.id)
        node_cam_db_obj = await self._session.execute(query)
        return [(int(node_id), int(cam_id)) for node_id, cam_id in node_cam_db_obj]
